//-------------------------------------------------------------------------
// This Edit macro performs two functions:
//
//    * Insert prefix string in front of all lines in current selection.
//      Prefix is not inserted if line already has prefix.
//
//    * Remove prefix string from front of all lines in current selection.
//      Prefix is only removed when present.
//
// Use mouse or ZZ line command to select block of lines before invoking
// this macro.
//
// This macro requires two parameters:   on | off   ...prefix-string...
//
// Example using "C" comment as prefix:
//
//    Insert prefix "//" on selected lines:   prefix on  //
//
//    Remove prefix "//" on selected lines:   prefix off //
//
// Some prefix strings may contain characters that are reserved for
// other purposes. Example, the semicolon(;) is the SPF primary
// command separator character. In this situation, enclose the prefix
// string in single or double quotes.
//
// Example using semicolon(;) as prefix:
//
//    prefix on ";"
//
//--------------------------------------------------------------------------
void prefix(int argc, char *argv[]) {

  char *pcmd;        // primary command string
  char *prefix_mode;
  char *select_mode;
  char *prefix;
  char *first_char;
  int   line_num_start;
  int   i;
  bool  any_error = FALSE;
  bool  prefix_quoted = FALSE;

  if (argc == 3) {
    prefix_mode = argv[1]; // get prefix mode parameter (on|off)
    strupr(prefix_mode);   // make mode all uppercase
    if (prefix_mode == "ON" || prefix_mode == "OFF") {
      line_num_start = spfservice("query","select_start_line");
      if (line_num_start > 0) {
        select_mode = spfservice("query","select_mode");
        if (select_mode == "WHOLELINE") {
          first_char = strgetsubstr(argv[2],1,1);
          if (first_char == "\"") // double quote?
            prefix_quoted = TRUE;
          if (first_char == "\'") // single quote?
            prefix_quoted = TRUE;
          if (prefix_quoted) {
            // prefix is quoted
            i = strlen(argv[2]); // length of raw prefix
            if (i > 2) {         // long enough for quotes and at least one char
              i -= 2;            // length minus quotes
              prefix = strgetsubstr(argv[2],2,i); // get quoted chars
              }
            else {
              any_error = TRUE;
              msg("PREFIX:  prefix has syntax error.");
            }
            }
          else {
            prefix = argv[2];
          }
          if (!any_error) {
            if (prefix_mode == "ON") {
              prefixon(prefix);
              }
            else {
              prefixoff(prefix);
            }
          }
          }
        else {
          msg("PREFIX:  Select whole lines, not stream or rectangle.");
        }
        }
      else {
        msg("PREFIX:  Use mouse or ZZ line command to select block of lines before invoking this macro.");
      }
      }
    else {
      msg("PREFIX: first parameter must be ON or OFF.");
    }
    }
  else {
    msg("PREFIX:  macro requires two parameters:   on | off   ...prefix-string...");
  }

} // prefix

//---------
// prefixon
//---------
void prefixon(char *prefix) {

  char *pcmd;           // primary command string
  char *original_line;
  char *modified_line;
  int   line_num_start, line_num_end, line_num_current;
  int   prefix_len;

  line_num_start = spfservice("query","select_start_line");
  line_num_end = spfservice("query","select_end_line");
  line_num_current = line_num_start;
  prefix_len = strlen(prefix);
  while (line_num_current <= line_num_end) {
    // position to select line
    pcmd = strcompose("LOCATE ",line_num_current);
    spfservice("cmd",pcmd);
    original_line = spfservice("query","line_data_raw");
    if (strncmp(prefix,original_line,prefix_len) != 0) { // prefix not already on line?
      // insert the prefix string
      strcpy(modified_line,prefix);
      strcat(modified_line,original_line);
      spfservice("set","line_data_raw",modified_line);
    }
    ++line_num_current; // go to next select line
  } // while
  pcmd = strcompose("LOCATE ",line_num_start); // pos to 1st line of selection
  spfservice("cmd",pcmd);
  spfservice("cmd","RESET");                   // reset selection

} // prefixon

//----------
// prefixoff
//----------
void prefixoff(char *prefix) {

  char *pcmd;          // primary command string
  char *dquote = "\""; // double quote
  int   line_num_start, line_num_end, line_num_current;

  line_num_start = spfservice("query","select_start_line");
  line_num_end = spfservice("query","select_end_line");
  line_num_current = line_num_start;
  while (line_num_current <= line_num_end) {
    // position to select line
    pcmd = strcompose("LOCATE ",line_num_current);
    spfservice("cmd",pcmd);
    // remove the prefix string
    strcpy(pcmd,"CHANGE ");
    if (prefix == "\'") { // is prefix a single quote?
      strcat(pcmd,dquote);
      strcat(pcmd,prefix);
      strcat(pcmd,dquote);
      }
    else {
      strcat(pcmd,"'");
      strcat(pcmd,prefix);
      strcat(pcmd,"'");
    }
    strcat(pcmd," '' 1 quiet");
    spfservice("cmd",pcmd);
    ++line_num_current; // go to next select line
  } // while
  pcmd = strcompose("LOCATE ",line_num_start); // pos to 1st line of selection
  spfservice("cmd",pcmd);
  spfservice("cmd","RESET");                   // reset selection

} // prefixoff
